/*
 * Copyright 2006-2008 The FLWOR Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef ZORBA_BASHMODULE_FILEBUF_H
#define ZORBA_BASHMODULE_FILEBUF_H

#include <cstdio>
#include <iostream>

namespace zorba{

class FileBuf : public std::streambuf
{
public:
  FileBuf(FILE*);
  FILE* getFILE();

protected:
  virtual int overflow(int c = EOF);
  virtual int underflow();
  virtual int uflow();
  virtual int pbackfail(int c = EOF);
  virtual int sync();
  virtual std::streampos seekoff (std::streamoff off, std::ios_base::seekdir way, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);

private:
  FILE* fptr;
};

FileBuf::FileBuf(FILE* f)
  : std::streambuf(), fptr(f) {}

int FileBuf::overflow(int c)
{
  return c != EOF ? fputc(c, fptr) : EOF;
}

int FileBuf::underflow()
{
  int c = getc(fptr);
  if (c != EOF)
  {
    ungetc(c, fptr);
  }
  return c;
}

int FileBuf::uflow()
{
  return getc(fptr);
}

int FileBuf::pbackfail(int c)
{
  return c != EOF ? ungetc(c, fptr) : EOF;
}

int FileBuf::sync()
{
  return fflush(fptr);
}

FILE* FileBuf::getFILE()
{
  return fptr;
}

std::streampos FileBuf::seekoff(std::streamoff off, std::ios_base::seekdir way, std::ios_base::openmode which)
{
  return 0;
}

}

#endif // ZORBA_BASHMODULE_FILEBUF_H
